@@ -89,7 +89,7 @@ void gd_struct_set_defaults_from_array(void *str, const GdStructDescriptor *prop
 #define GD_NUM_OF_CELLS		(GD_NUM_OF_CELLS_X * GD_NUM_OF_CELLS_Y + 80)
 
 // maximum replay size (maximum seconds x game cycles per second)
-#define MAX_REPLAY_LEN		(10000 * FRAMES_PER_SECOND / 8)
+#define MAX_REPLAY_LEN		(20000 * FRAMES_PER_SECOND / 8)
 
 extern const GdColor gd_flash_color;
 extern const GdColor gd_select_color;
@@ -287,6 +287,12 @@ typedef struct _replay_movements
   unsigned int len;
 } GdReplayMovements;
 
+typedef struct _replay_randoms
+{
+  unsigned int data[MAX_REPLAY_LEN];
+  unsigned int len;
+} GdReplayRandoms;
+
 // maximum seed value for the cave random generator. should be smaller than a signed int.
 #define GD_CAVE_SEED_MAX	(1 << 30)
 
@@ -308,6 +314,7 @@ typedef struct _gd_cave_replay
 
   boolean wrong_checksum;
   GdReplayMovements *movements;
+  GdReplayRandoms *randoms;
   int current_playing_pos;
 } GdReplay;
 
@@ -373,8 +380,13 @@ typedef struct _gd_cave
                                         // but can be other
   boolean short_explosions;             // in >= 1stb, diamond/creature explosions were of 5 stages
 
+  boolean use_krissz_engine;            // for game engine compatibility with Krissz engine
+  boolean new_krissz_engine;            // for game engine compatibility with Krissz engine (new)
+  GdString krissz_engine_cave_code;     // internal cave code for Krissz engine caves
+
   GdScheduling scheduling;              // scheduling type; see above
   boolean pal_timing;                   // use faster seconds
+  boolean no_time;                      // use no time at all for this level
 
   boolean active_is_first_found;        // active player is the uppermost.
   boolean lineshift;                    // true is line shifting emulation,
@@ -383,6 +395,9 @@ typedef struct _gd_cave
                                         // false for plck
   boolean wraparound_objects;           // if this is true, object drawing (cave rendering)
                                         // will wraparound as well.
+  boolean open_borders_horizontal;      // cave is open for crossing borders horizontally
+  boolean open_borders_vertical;        // cave is open for crossing borders vertically
+  boolean infinite_scrolling;           // use scrolling instead of wrapping at playfield borders
 
   GdElement initial_fill;
   GdElement initial_border;
@@ -400,10 +415,9 @@ typedef struct _gd_cave
 
   int w, h;                             // Sizes of cave, width and height.
   int x1,y1,x2,y2;                      // Visible part of the cave
-  GdColor colorb;                       // border color
-  GdColor color0, color1, color2, color3, color4, color5;    // c64-style colors;
-                                                             // color 4 and 5 are amoeba and slime.
-  GdColor color6, color7;                                    // extra colors
+  GdColor color_b;                      // border color
+  GdColor color[MAX_LEVEL_COLORS];      // c64-style colors; color 4 and 5 are amoeba and slime.
+  GdColor base_color[MAX_LEVEL_COLORS]; // base colors for color gradients
   int diamond_value;                    // Score for a diamond.
   int extra_diamond_value;              // Score for a diamond, when gate is open.
 
@@ -466,6 +480,8 @@ typedef struct _gd_cave
                                             // for example diamond -> falling diamond
   GdElement slime_eats_2, slime_converts_2; // this is usually stone -> stone_f
   GdElement slime_eats_3, slime_converts_3; // this is usually nut -> nut_f
+  GdElement slime_eats_4, slime_converts_4; // this is usually empty (not defined)
+  GdElement slime_eats_5, slime_converts_5; // this is usually empty (not defined)
   boolean slime_sound;                  // slime has sound
 
   boolean lava_sound;                   // elements sinking in lava have sound
@@ -642,6 +658,7 @@ typedef struct _gd_cave
   boolean inbox_flash_toggle;           // negated every scan. helps drawing inboxes, and making
                                         // players be born at different times.
   GdDirection last_direction;           // last direction player moved. used by draw routines
+  GdDirection last_direction_2nd;       // 2nd last direction player moved. used by draw routines
   GdDirection last_horizontal_direction;
   int biters_wait_frame;                // number of frames to wait until biters will move again
   int replicators_wait_frame;           // number of frames to wait until replicators are
@@ -739,7 +756,7 @@ void gd_drawcave_game(const GdCave *cave,
 		      boolean bonus_life_flash, int animcycle, boolean hate_invisible_outbox);
 
 // function to copy a GdString
-static inline char *gd_strcpy(GdString dest, const GdString src)
+static inline char *gd_strcpy(GdString dest, const char *src)
 {
     return strncpy(dest, src, sizeof(GdString));
 }
@@ -750,6 +767,7 @@ GdReplay *gd_replay_new(void);
 GdReplay *gd_replay_new_from_replay(GdReplay *orig);
 void gd_replay_free(GdReplay *replay);
 void gd_replay_store_movement(GdReplay *replay, GdDirection player_move, boolean player_fire, boolean suicide);
+void gd_replay_store_random(GdReplay *replay, int random);
 
 unsigned int gd_cave_adler_checksum(GdCave *cave);
 void gd_cave_adler_checksum_more(GdCave *cave, unsigned int *a, unsigned int *b);
