@@ -97,6 +97,11 @@ const GdStructDescriptor gd_caveset_properties[] =
     N_("Bonus life score"), CAVESET_OFFSET(bonus_life_score), 1,
     N_("Number of points to collect for a bonus life."), 100, 5000
   },
+  {
+    "KrisszEngine", GD_TYPE_BOOLEAN, 0,
+    N_("Use Krissz game engine"), CAVESET_OFFSET(use_krissz_engine), 1,
+    N_("Enable compatibility with the Krissz game engine.")
+  },
   {
     "Story", GD_TYPE_LONGSTRING, 0,
     N_("Story"), CAVESET_OFFSET(story), 1,
@@ -130,6 +135,8 @@ static GdPropertyDefault caveset_defaults[] =
   { CAVESET_OFFSET(maximum_lives),	9	},
   { CAVESET_OFFSET(bonus_life_score),	500	},
 
+  { CAVESET_OFFSET(use_krissz_engine),	FALSE	},
+
   { -1 },
 };
 
@@ -142,7 +149,15 @@ GdCavesetData *gd_caveset_data_new(void)
   gd_struct_set_defaults_from_array(data, gd_caveset_properties, caveset_defaults);
 
   if (leveldir_current != NULL)
+  {
+    data->initial_lives = leveldir_current->bd_initial_lives;
+    data->maximum_lives = leveldir_current->bd_maximum_lives;
+    data->bonus_life_score = leveldir_current->bd_bonus_life_score;
+
+    data->use_krissz_engine = leveldir_current->bd_use_krissz_engine;
+
     data->levelset_subdir = getStringCopy(leveldir_current->subdir);
+  }
 
   return data;
 }
@@ -170,7 +185,7 @@ void gd_caveset_clear(void)
 {
   if (gd_caveset)
   {
-    list_foreach(gd_caveset, (list_fn) gd_cave_free, NULL);
+    list_foreach_fn_1(gd_caveset, (list_fn_1) gd_cave_free);
     list_free(gd_caveset);
     gd_caveset = NULL;
   }
@@ -239,6 +254,10 @@ GdCave *gd_get_prepared_cave_from_caveset(const int cave, const int level)
   // get prepared cave from original cave
   GdCave *prepared_cave = gd_get_prepared_cave(original_cave, level);
 
+  // Krissz engine: set flag to use Krissz engine in cave if set in cave set
+  if (gd_caveset_data->use_krissz_engine)
+    prepared_cave->use_krissz_engine = TRUE;
+
   return prepared_cave;
 }
 
@@ -502,15 +521,15 @@ static void brc_import(byte *data)
       // colors
       colind = data[31 * c + datapos] % ARRAY_SIZE(brc_color_table);
 
-      cave->colorb = 0x000000;    // fixed rgb black
-      cave->color0 = 0x000000;    // fixed rgb black
-      cave->color1 = brc_color_table[colind];         // brc specified dirt color
-      cave->color2 = brc_color_table_comp[colind];    // complement
-      cave->color3 = 0xffffff;    // white for brick
-      cave->color4 = 0xe5ad23;    // fixed for amoeba
-      cave->color5 = 0x8af713;    // fixed for slime
-      cave->color6 = 0x888888;    // extra color 1 - gray
-      cave->color7 = 0xffffff;    // extra color 2 - white
+      cave->color_b  = 0x000000;    // fixed rgb black
+      cave->color[0] = 0x000000;    // fixed rgb black
+      cave->color[1] = brc_color_table[colind];         // brc specified dirt color
+      cave->color[2] = brc_color_table_comp[colind];    // complement
+      cave->color[3] = 0xffffff;    // white for brick
+      cave->color[4] = 0xe5ad23;    // fixed for amoeba
+      cave->color[5] = 0x8af713;    // fixed for slime
+      cave->color[6] = 0x888888;    // extra color 1 - gray
+      cave->color[7] = 0xffffff;    // extra color 2 - white
 
       if (import_effect)
       {
@@ -599,22 +618,22 @@ static void caveset_name_set_from_filename(char *filename)
 */
 boolean gd_caveset_load_from_file(char *filename)
 {
+  size_t filesize = getSizeOfFile(filename);
   size_t length;
   char *buf;
   List *new_caveset;
-  struct stat st;
   File *file;
 
-  if (stat(filename, &st) != 0)
+  if (filesize < 0)
   {
-    Warn("cannot stat() file");
+    Warn("cannot get size of file '%s'", filename);
 
     return FALSE;
   }
 
-  if (st.st_size > 1048576)
+  if (filesize > GD_CAVESET_MAX_FILESIZE)
   {
-    Warn("file bigger than 1MiB, refusing to load");
+    Warn("cannot load file '%s' (bigger than %d bytes)", filename, GD_CAVESET_MAX_FILESIZE);
 
     return FALSE;
   }
@@ -626,13 +645,13 @@ boolean gd_caveset_load_from_file(char *filename)
     return FALSE;
   }
 
-  buf = checked_malloc(st.st_size + 1);
-  length = readFile(file, buf, 1, st.st_size);
+  buf = checked_malloc(filesize + 1);
+  length = readFile(file, buf, 1, filesize);
   buf[length] = '\0';
 
   closeFile(file);
 
-  if (length < st.st_size)
+  if (length < filesize)
   {
     Warn("cannot read file '%s'", filename);
 
